#!/usr/bin/perl
#
# Tiny link database.
#
# This software is under Public Domain
#
# Requirements
#       Perl 5.005 or later (maybe) and flock
#       CGI::Minimal (Tested with 1.29)
#       CGI::FastTemplate (Tested with 1.09)
#       Digest::MD5
#

use strict;
#use warnings;

use lib qw(lib); # library path

use vars qw(
$C_THIS_URL $C_MAX_QUERY $C_MAX_KEYWORD_LENGTH $C_MAX_URL_LENGTH
$C_WAIT $C_DB_PREFIX $C_DB_SUFFIX $C_DB_HASH_LENGTH $C_DB_MAX

$P_KEY $P_URL

$T_FORM $T_JUMP $T_RESULT $T_LINK

$M_DEFAULT $M_ERROR_REQUEST $M_ERROR_KEY $M_ERROR_URL $M_ERROR_LOCK
$M_ERROR_READ $M_ERROR_WRITE $M_REGISTER $M_REGISTER_SUCCESS
$M_REGISTER_EXIST
);

# configulation
#

# URL of this script.
# This is used to make a link refers to this script.
$C_THIS_URL = 'links.cgi';
# Maximum query size
$C_MAX_QUERY = 1024;
# Maximum length for keyword.
$C_MAX_KEYWORD_LENGTH = 256;
# Maximum length for URL.
$C_MAX_URL_LENGTH = 256;
# Wait to refresh (seconds).
$C_WAIT = 10;
# Prefix of detabase file.
$C_DB_PREFIX = 'data/links';
# Suffix of detabase file.
$C_DB_SUFFIX = '.cgi';
# 
$C_DB_HASH_LENGTH = 2;
# Maximum records per a database file.
$C_DB_MAX = 16;

# Maximum records is calculated by this.
# 16 ** $C_DB_HASH_LENGTH * $C_DB_MAX
# Required disk space is calculated by this.
# (Maximum records) * (3 * $C_MAX_KEYWORD_LENGTH + $C_MAX_URL_LENGTH + 2)

# parameters
#

# Parameter name of keyword
$P_KEY = 'k';
# Parameter name of URL
$P_URL = 'u';

# templates
#
$T_FORM = <<'EOT';
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>Bookmark</title></head>
<body>
<p>$MESSAGE</p>$LINK
<form method="post" action="$C_THIS_URL" enctype="application/x-www-form-urlencoded" accept-charset="utf-8">
<p>Keyword <input type="text" name="$P_KEY" value="$KEY" tabindex="1" size="60" maxlength="256" accesskey="1" /></p>
<p>URL (http, https, ftp only) <input type="text" name="$P_URL" value="$URL" tabindex="2" size="60" maxlength="256" accesskey="2" /></p>
<p><input type="submit" tabindex="3" name="submit" accesskey="3" />
<input type="reset" tabindex="4" name="reset" accesskey="4" /></p></form>
<p><a href="/">Back to main page.</a></p>
</body>
</html>
EOT
$T_JUMP = <<'EOT';
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<meta http-equiv="Refresh" content="$WAIT;url=$URL" />
<title>Bookmark</title></head>
<body>
<p><a href="$URL">$URL</a></p>
</body>
</html>
EOT
$T_RESULT = <<'EOT';
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>Bookmark</title>
</head>
<body>
<p>$MESSAGE</p>
<p><a href="$C_THIS_URL">Back to form.</a></p>
<p><a href="/">Back to main page.</a></p>
</body>
</html>
EOT
$T_LINK = '<p><a href="$URL">$URL</a></p>';

# Mesages
#
$M_DEFAULT = 'This presents a URL associated with a keyword that you give. Also this can accept a pair of URL and keyword to register. The pair might be expired after a while.';
$M_ERROR_REQUEST = 'Invalid request';
$M_ERROR_KEY = 'The keyword is too long.';
$M_ERROR_URL = 'The URL is invalid.';
$M_ERROR_LOCK = "LOCK ERROR";
$M_ERROR_READ = "READ ERROR";
$M_ERROR_WRITE = "WRITE ERROR";
$M_REGISTER = "The keyword isn't registered. You can register URL with the keyword.";
$M_REGISTER_SUCCESS = 'Registered.';
$M_REGISTER_EXIST = 'The keyword is already registered.';

require 'links.pl';
