use strict;
#use warnings;

# Program

use CGI::Minimal;
use CGI::FastTemplate;
use Digest::MD5 qw(md5_hex);
CGI::Minimal::reset_globals;
CGI::Minimal::max_read_size($C_MAX_QUERY);

binmode(STDIN);
binmode(STDOUT);

&dispatch;

sub dispatch {
	my $q = new CGI::Minimal;
	if ($q->truncated()) {
		&show_result($q, $M_ERROR_REQUEST);
	}
	my $key = &parameter_exist($q->param($P_KEY));
	my $url = &parameter_exist($q->param($P_URL));
	# validate
	if ($key && ! &check_keyword($q->param($P_KEY))) {
		&show_result($q, $M_ERROR_KEY);
	}
	if ($url && ! &check_url($q->param($P_URL))) {
		&show_result($q, $M_ERROR_URL);
	}
	local *LFH;
	# dispatch
	if ($key) {
		if ($url) {
			&do_register($q);
		} else {
			&do_jump($q);
		}
	} elsif ($url) {
		&show_jump($q, $q->param($P_URL));
	} else {
		&show_form($q, $M_DEFAULT);
	}
}

sub parameter_exist { defined $_[0] && length $_[0] > 0; }
sub check_keyword { length $_[0] <= $C_MAX_KEYWORD_LENGTH; }
sub check_url {
	length $_[0] <= $C_MAX_URL_LENGTH &&
	$_[0] =~ m{^(?:http|https|ftp)://(?:[a-zA-Z0-9\41\43\44\46\47\50-\57\72\73\75\77\100\137\176]|\45[0-9a-fA-F][0-9a-fA-F])+$}o;
}

sub do_register {
	my $q = shift;
	$ENV{'REQUEST_METHOD'} eq 'POST' or &show_result($q, $M_ERROR_REQUEST);
	my ($n, $tn) = &get_filename($q->param($P_KEY));
	my $k = $q->url_encode($q->param($P_KEY));
	&do_lock($q, $n, 2);
	my ($f, @keys);
	my $u = "$C_THIS_URL?$P_KEY=$k";
	if (open($f, $n)) {
		while (<$f>) {
			chomp;
			my @l = split("\t");
			if ($l[0] eq $k) {
				close($f);
				&show_form($q, $M_REGISTER_EXIST, $q->param($P_KEY),
					$q->param($P_URL), $u);
			}
			push(@keys, $_);
		}
		close($f);
	}
	unshift(@keys, $k."\t".$q->param($P_URL));
	splice(@keys, $C_DB_MAX) if ($#keys >= $C_DB_MAX);
	open($f, ">$tn") or &show_result($q, $M_ERROR_WRITE);
	my $result = 0;
	do {
		local $, = "\n"; local $\ = "\n";
		$result = print $f @keys;
	};
	close($f);
	($result && rename($tn, $n)) or &show_result($q, $M_ERROR_WRITE);
	&show_form($q, $M_REGISTER_SUCCESS, '', '', $u);
}

sub do_jump {
	my $q = shift;
	my ($n, $tn) = &get_filename($q->param($P_KEY));
	my $k = $q->url_encode($q->param($P_KEY));
	&do_lock($q, $n, 1);
	unlink($tn);
	my $f;
	if (open($f, $n)) {
		while (<$f>) {
			chomp;
			my @l = split("\t");
			if ($l[0] eq $k) {
				close($f);
				if (&check_url($l[1])) {
					&show_jump($q, $l[1]);
				} else {
					&show_result($q, $M_ERROR_READ);
				}
			}
		}
		close($f);
	}
	&show_form($q, $M_REGISTER, $q->param($P_KEY));
}

sub get_filename {
	my $name = $C_DB_PREFIX.substr(md5_hex($_[0]), 0, $C_DB_HASH_LENGTH).$C_DB_SUFFIX;
	$name, "$name.tmp";
}

sub show_form {
	my $q = shift;
	my $t = new CGI::FastTemplate();
	$t->define_nofile(main => $T_FORM, anchor => $T_LINK);
	if (defined $_[3]) {
		$t->assign(URL => $q->htmlize($_[3]));
		$t->parse(LINK => 'anchor');
		$t->clear('URL');
	} else {
		$t->assign(LINK => '');
	}
	$t->assign(
		C_THIS_URL => $C_THIS_URL,
		P_KEY => $P_KEY,
		P_URL => $P_URL,
		MESSAGE => $q->htmlize(defined $_[0] ? $_[0] : ''),
		KEY => $q->htmlize(defined $_[1] ? $_[1] : ''),
		URL => $q->htmlize(defined $_[2] ? $_[2] : ''),
	);
	$t->parse(MAIN => 'main');
	&print_header();
	$t->print();
	$t->clear();
	&do_unlock;
	exit;
}

sub show_jump {
	my $q = shift;
	my $t = new CGI::FastTemplate();
	$t->define_nofile(main => $T_JUMP);
	$t->assign(WAIT => int($C_WAIT), URL => $q->htmlize($_[0]));
	$t->parse(MAIN => 'main');
	&print_header();
	$t->print();
	$t->clear();
	&do_unlock;
	exit;
}

sub show_result {
	my $q = shift;
	my $t = new CGI::FastTemplate();
	$t->define_nofile(main => $T_RESULT);
	$t->assign(C_THIS_URL => $C_THIS_URL, MESSAGE => $q->htmlize($_[0]));
	$t->parse(MAIN => 'main');
	&print_header();
	$t->print();
	$t->clear();
	&do_unlock;
	exit;
}

sub print_header {
	print
		"Content-Type: text/html\r\n",
		"Cache-Control: no-cache\r\n",
		"Pragma: no-cache\r\n\r\n";
}

sub do_lock {
	my $q = shift;
	if (open(LFH, '+>>'.$_[0].'.lock')) {
		if (flock(LFH, $_[1])) {
			return;
		}
		close(LFH);
	}
	&show_result($q, $M_ERROR_LOCK);
}

sub do_unlock {
	close(LFH) if (defined(fileno(LFH)));
}

1;
